@@ -1,4 +1,5 @@
-# Copyright 2001-2004 The Apache Software Foundation
+# Copyright 2001-2006 The Apache Software Foundation or its licensors, as
+# applicable.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -346,6 +347,11 @@ my @cmodule_config_defines = map {
     cmodule_define($_);
 } @cmodule_config_names;
 
+my $cmodule_config_extra = 
+    "#ifndef APACHE_HTTPD_TEST_EXTRA_HOOKS\n".
+    "#define APACHE_HTTPD_TEST_EXTRA_HOOKS(p) do { } while (0)\n".
+    "#endif\n";
+
 my $cmodule_config_hooks = join ",\n    ", map {
     cmodule_define_name($_);
 } @cmodule_config_names;
@@ -396,6 +402,7 @@ my $cmodule_template_2 = <<"EOF";
 static void name ## _register_hooks(apr_pool_t *p)
 {
 @cmodule_hooks
+    APACHE_HTTPD_TEST_EXTRA_HOOKS(p);
 }
 
 module AP_MODULE_DECLARE_DATA name ## _module = {
@@ -435,6 +442,8 @@ sub cmodules_generate_include {
 
     print $fh @cmodule_hook_defines, @cmodule_config_defines;
 
+    print $fh $cmodule_config_extra;
+
     print $fh $self->cmodules_module_template;
 
     close $fh;
@@ -460,6 +469,8 @@ __DATA__
 #define AP_METHOD_BIT  1
 typedef size_t apr_size_t;
 typedef array_header apr_array_header_t;
+#define APR_OFF_T_FMT "ld"
+#define APR_SIZE_T_FMT "lu"
 #endif /* APACHE1 */
 
 #ifdef APACHE2
