@@ -1,4 +1,5 @@
-# Copyright 2001-2004 The Apache Software Foundation
+# Copyright 2001-2006 The Apache Software Foundation or its licensors, as
+# applicable.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -144,17 +145,35 @@ EOF
     close $fh or die "close $t: $!";
 }
 
-# propogate trace overrides to the server
-sub configure_trace {
+# propogate PerlPassEnv settings to the server
+sub configure_env {
     my $self = shift;
-    $self->postamble(IfModule => 'mod_perl.c',
-                     "PerlPassEnv APACHE_TEST_TRACE_LEVEL\n");
+    $self->preamble(IfModule => 'mod_perl.c', 
+                    [ qw(PerlPassEnv APACHE_TEST_TRACE_LEVEL
+                         PerlPassEnv HARNESS_PERL_SWITCHES)
+                    ]);
 }
 
 sub startup_pl_code {
     my $self = shift;
     my $serverroot = $self->{vars}->{serverroot};
 
+    my $cover = <<'EOF';
+    if (($ENV{HARNESS_PERL_SWITCHES}||'') =~ m/Devel::Cover/) {
+        eval {
+            # 0.48 is the first version of Devel::Cover that can
+            # really generate mod_perl coverage statistics
+            require Devel::Cover;
+            Devel::Cover->VERSION(0.48);
+
+            # this ignores coverage data for some generated files
+            Devel::Cover->import('+inc' => 't/response/',);
+
+            1;
+        } or die "Devel::Cover error: $@";
+    }
+EOF
+
     return <<"EOF";
 BEGIN {
     use lib '$serverroot';
@@ -162,6 +181,8 @@ BEGIN {
         eval { require "conf/\$file" } or
             die if grep { -e "\$_/conf/\$file" } \@INC;
     }
+
+$cover
 }
 
 1;
@@ -184,8 +205,6 @@ sub configure_startup_pl {
         if (-d $tlib) {
             print $fh "use lib '$tlib';\n";
         }
-        my $fixup = Apache::TestConfig->modperl_2_inc_fixup();
-        print $fh $fixup;
 
         # if Apache::Test is used to develop a project, we want the
         # project/lib directory to be first in @INC (loaded last)
