@@ -1,4 +1,5 @@
-# Copyright 2001-2004 The Apache Software Foundation
+# Copyright 2001-2006 The Apache Software Foundation or its licensors, as
+# applicable.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -50,9 +51,15 @@ sub clean {
 }
 
 sub test {
-
+    my $self = shift;
     my $env = Apache::TestConfig->passenv_makestr();
 
+    my $tests = "TEST_FILES =\n";
+
+    if (ref $self && exists $self->{'test'}) {
+        $tests = 'TEST_FILES = ' . $self->{'test'}->{'TESTS'} . "\n";
+    }
+
     my $preamble = Apache::TestConfig::WIN32 ? "" : <<EOF;
 PASSENV = $env
 EOF
@@ -62,14 +69,14 @@ EOF
     if (eval { require Devel::Cover }) {
         my $atdir = File::Spec->catfile($ENV{HOME}, '.apache-test');
 
-        $cover = <<"EOF"
+        my $cover_exec = Apache::TestConfig::which("cover");
 
-testcover :
-	-\@cover -delete
-	-HARNESS_PERL_SWITCHES=-MDevel::Cover=+inc,$atdir \\
-	APACHE_TEST_EXTRA_ARGS=-one-process \$(MAKE) test
-	-\@cover
-EOF
+        my @cover = ("", "testcover :", );
+        push @cover, "\t-\@$cover_exec -delete" if $cover_exec;
+        push @cover, "\t-HARNESS_PERL_SWITCHES=-MDevel::Cover=+inc,$atdir \\",
+            "\tAPACHE_TEST_EXTRA_ARGS=-one-process \$(MAKE) test";
+        push @cover, "\t-\@$cover_exec" if $cover_exec;
+        $cover = join "\n", @cover, "";
     }
     else {
 
@@ -77,12 +84,12 @@ EOF
 
 testcover :
 	@echo "Cannot run testcover action unless Devel::Cover is installed"
+	@echo "Don't forget to rebuild your Makefile after installing Devel::Cover"
 EOF
     }
 
-    return $preamble . <<'EOF' . $cover;
+    return $preamble . $tests . <<'EOF' . $cover;
 TEST_VERBOSE = 0
-TEST_FILES =
 
 test_clean :
 	$(FULLPERL) -I$(INST_ARCHLIB) -I$(INST_LIB) \
@@ -244,7 +251,6 @@ our example it'll look for F<t/TEST.PL>. The autogenerated script
 F<t/TEST> will include the contents of F<t/TEST.PL>, and special
 directives, including any configuration options passed via
 C<L<filter_args()|/C_filter_args_>> called from F<Makefile.PL>, special
-fixup code, etc. If no argument is passed to C<generate_script()>,
-it will create a file named F<t/TEST> by default.
+fixup code, etc.
 
 =cut
