@@ -1,4 +1,5 @@
-# Copyright 2001-2004 The Apache Software Foundation
+# Copyright 2001-2006 The Apache Software Foundation or its licensors, as
+# applicable.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -67,14 +68,8 @@ sub post_config {
     # string and various variations made by distributions which mangle
     # that string
 
-    # Apache/2.0.50-dev
-    ($self->{rev})   = $self->{version} =~ m|^Apache/(\d)\.|;
-
-    # Apache-AdvancedExtranetServer/1.3.29 (Mandrake Linux/1mdk)
-    ($self->{rev}) ||= $self->{version} =~ m|^Apache.*?/(\d)\.|;
-
-    # IBM_HTTP_SERVER/1.3.19  Apache/1.3.20 (Unix)
-    ($self->{rev}) ||= $self->{version} =~ m|^.*?Apache.*?/(\d)\.|;
+    # Foo-Apache-Bar/x.y.z
+    ($self->{rev}) = $self->{version} =~ m|/(\d)\.|;
 
     if ($self->{rev}) {
         debug "Matched Apache revision $self->{version} $self->{rev}";
@@ -105,6 +100,7 @@ my @apache_logs = qw(
 error_log access_log httpd.pid
 apache_runtime_status rewrite_log
 ssl_engine_log ssl_request_log
+cgisock
 );
 
 sub clean {
@@ -122,7 +118,7 @@ sub clean {
 
 sub pid_file {
     my $self = shift;
-    catfile $self->{config}->{vars}->{t_logs}, 'httpd.pid';
+    $self->{config}->{vars}->{t_pid_file};
 }
 
 sub dversion {
